# A program which manages Ubuntu-it's web test server
# Copyright (C) 2016  Pietro Albini <pietroalbini@ubuntu.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; witout even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import traceback
import threading
import queue


class Processor(threading.Thread):
    """This processes the supplied tasks"""

    def __init__(self):
        self._queue = queue.Queue()

        super().__init__()

    def queue(self, job):
        """Queue a new job"""
        if not callable(job):
            raise RuntimeError("Job not callable: %r" % job)

        self._queue.put(job)

    def stop(self):
        """Stop the processor"""
        self._queue.put(None)
        self.join()

    def run(self):
        """Run the thread"""
        while True:
            job = self._queue.get()

            # This is the stop signal
            if job is None:
                break

            # Execute the job
            try:
                job()
            except:
                traceback.print_exc()
